@@ -1,6 +1,9 @@
 #### "Namespace private" Auxiliaries  such as method functions
 #### (called from more than one place --> need to be defined early)
 
+.Matrix.avoiding.as.matrix <- FALSE # (always on CRAN -- have documented it since 2015)
+## NB: sync with  ../NAMESPACE
+
 ## Need to consider NAs ;  "== 0" even works for logical & complex:
 ## Note that "!x" is faster than "x == 0", but does not (yet!) work for complex
 ## if we did these in C, would gain a factor 2 (or so):
@@ -9,23 +12,24 @@ isN0 <- function(x)  is.na(x) | x != 0
 is1  <- function(x) !is.na(x) & x   # also == "isTRUE componentwise"
 
 ##
+##allFalse <- function(x) !any(x) && !any(is.na(x))## ~= all0, but allFalse(NULL) = TRUE w/warning
 ##all0 <- function(x) !any(is.na(x)) && all(!x) ## ~= allFalse
-all0 <- function(x) .Call(R_all0, x)
+allFalse <- function(x) if(is.atomic(x)) .Call(R_all0, x) else !any(x) && !any(is.na(x))
+all0     <- function(x) if(is.atomic(x)) .Call(R_all0, x) else all(!x) && !any(is.na(x))
 
+##anyFalse <- function(x) isTRUE(any(!x))		 ## ~= any0
 ## any0 <- function(x) isTRUE(any(x == 0))	      ## ~= anyFalse
-any0 <- function(x) .Call(R_any0, x)
+anyFalse <-
+any0 <- function(x) if(is.atomic(x)) .Call(R_any0, x) else isTRUE(any(!x))
 
 ## These work "identically" for  1 ('==' TRUE)  and 0 ('==' FALSE)
 ##	(but give a warning for "double"  1 or 0)
 ## TODO: C versions of these would be faster
-allTrue  <- function(x) all(x)  && !any(is.na(x))
+allTrue  <- function(x) all(x) && !anyNA(x)
 
-##allFalse <- function(x) !any(x) && !any(is.na(x))## ~= all0, but allFalse(NULL) = TRUE w/warning
-allFalse <- function(x) .Call(R_all0, x)
-
-##anyFalse <- function(x) isTRUE(any(!x))		 ## ~= any0
-anyFalse <- function(x) .Call(R_any0, x)
 
+## Note that mode(<integer>) = "numeric" -- as0(), as1() return "double"
+## which is good *AS LONG AS* we do not really have i..Matrix integer matrices
 as1 <- function(x, mod=mode(x))
     switch(mod, "integer"= 1L, "double"=, "numeric"= 1, "logical"= TRUE,
 	   "complex"= 1+0i, stop(gettextf("invalid 'mod': %s", mod), domain = NA))
@@ -33,8 +37,19 @@ as0 <- function(x, mod=mode(x))
     switch(mod, "integer"= 0L, "double"=, "numeric"= 0, "logical"= FALSE,
 	   "complex"= 0+0i, stop(gettextf("invalid 'mod': %s", mod), domain = NA))
 
+##' Should the matrix/Matrix  x  or a combination of x and y   be treated as  'sparse' ?
+## sparseDefault <- function(x, y=NULL) {
+##     if(is.null(y))
+##         prod(dim(x)) > 2*sum(isN0(as(x, "matrix")))
+##     else ## nrow / ncol ... differentiate  this would be for  rbind / cbind --> ./bind2.R
+##         (nnzero(x) + nnzero(y)) * 2 < (nrow(x)+nrow(y)) * nc
+## }
+sparseDefault <- function(x) prod(dim(x)) > 2*sum(isN0(as(x, "matrix")))
 
-.M.DN <- function(x) if(!is.null(dn <- dimnames(x))) dn else list(NULL,NULL)
+
+## NB:  .fixupDimnames() needs to be defined in ./AllClass.R
+
+.M.DN <- function(x) dimnames(x) %||% list(NULL,NULL)
 
 .has.DN <- ## has non-trivial Dimnames slot?
     function(x) !identical(list(NULL,NULL), x@Dimnames)
@@ -51,11 +66,15 @@ is.null.DN <- function(dn) {
     }
 }
 
-.if.NULL <- function(x, orElse) if(!is.null(x)) x else orElse
+##' return 'x' unless it is NULL where you'd use 'orElse'
+`%||%` <- function(x, orElse) if(!is.null(x)) x else orElse
 
 ##  not %in%  :
 `%nin%` <- function (x, table) is.na(match(x, table))
 
+nonTRUEoption <- function(ch) is.null(v <- getOption(ch)) || !isTRUE(v)
+
+
 ##' @title Check identical(i, 0:n) {or identical(i, 1:n) when Ostart is false}
 ##' @param i an integer vector, to be compared with 0:n or 1:n
 ##' @param n an integer number
@@ -63,19 +82,21 @@ is.null.DN <- function(dn) {
 ##' @return TRUE or FALSE
 ##' @author Martin Maechler
 isSeq <- function(i, n, Ostart = TRUE) {
-    ## FIXME: Port to C, use simple .Call() which is must faster notably in FALSE cases
+    ## FIXME: Port to C, use simple .Call() which is much faster notably in FALSE cases
     ##        and then *export* (and hence document)
     identical(i, if(Ostart) 0L:n else seq_len(n))
 }
 
 
 .bail.out.1 <- function(fun, cl) {
-    stop(gettextf('not-yet-implemented method for %s(<%s>).\n ->>  Ask the package authors to implement the missing feature.', fun, cl),
-	 call. = FALSE, domain=NA)
+    stop(gettextf(
+     'not-yet-implemented method for %s(<%s>).\n ->>  Ask the package authors to implement the missing feature.',
+		  fun, cl[1L]), call. = FALSE, domain=NA)
 }
 .bail.out.2 <- function(fun, cl1, cl2) {
-    stop(gettextf('not-yet-implemented method for %s(<%s>, <%s>).\n ->>  Ask the package authors to implement the missing feature.',
-		  fun, cl1, cl2), call. = FALSE, domain=NA)
+    stop(gettextf(
+     'not-yet-implemented method for %s(<%s>, <%s>).\n ->>  Ask the package authors to implement the missing feature.',
+		  fun, cl1[1L], cl2[1L]), call. = FALSE, domain=NA)
 }
 
 Matrix.msg <- function(..., .M.level = 1) {
@@ -83,6 +104,11 @@ Matrix.msg <- function(..., .M.level = 1) {
         message(...)
 }
 
+## TODO: faster via C, either R's  R_data_class() [which needs to become API !]
+##       or even direct  getAttrib(x, R_ClassSymbol); ..
+##' class - single string, no "package" attribute,..
+.class0 <- function(x)  as.vector(class(x))
+
 ## we can set this to FALSE and possibly measure speedup:
 .copyClass.check <- TRUE
 
@@ -102,20 +128,20 @@ copyClass <- function(x, newCl, sNames =
     r
 }
 
+##' Return the (maybe super-)class of class 'cl'   from "Matrix", returning  character(0) if there is none.
+##'
+##' @title The Matrix (Super-) Class of a Class
+##' @param cl string, class name
+##' @param cld its class definition
+##' @param ...Matrix if TRUE, the result must be of pattern "[dlniz]..Matrix"
+##'     where the first letter "[dlniz]" denotes the content kind.
+##' @param dropVirtual
+##' @param ... other arguments are passed to .selectSuperClasses()
+##' @return a character string
+##' @author Martin Maechler, Date: 24 Mar 2009
 MatrixClass <- function(cl, cld = getClassDef(cl),
 			...Matrix = TRUE, dropVirtual = TRUE, ...)
 {
-    ## Purpose: return the (maybe super-)class of class 'cl'   from "Matrix",
-    ##		returning  character(0) if there is none.
-    ## ----------------------------------------------------------------------
-    ## Arguments: cl: string, class name
-    ##		 cld: its class definition
-    ##	   ...Matrix: if TRUE, the result must be of pattern "[dlniz]..Matrix"
-    ##                where the first letter "[dlniz]" denotes the content kind.
-    ##	      ..... : other arguments are passed to .selectSuperClasses()
-    ## ----------------------------------------------------------------------
-    ## Author: Martin Maechler, Date: 24 Mar 2009
-
     ## stopifnot(is.character(cl))
     ## Hmm, packageSlot(cl)  *can* be misleading --> use  cld@package  first:
     if(is.null(pkg <- cld@package)) {
@@ -167,7 +193,7 @@ attr.all_Mat <- function(target, current,
 ## chol() via "dpoMatrix"
 ## This will only be called for *dense* matrices
 cholMat <- function(x, pivot = FALSE, ...) {
-    packed <- length(x@x) < prod(dim(x)) ## is it packed?
+    packed <- .isPacked(x)
     nmCh <- if(packed) "pCholesky" else "Cholesky"
     if(!is.null(ch <- x@factors[[nmCh]]))
 	return(ch) ## use the cache
@@ -227,7 +253,7 @@ detSparseLU <- function(x, logarithm = TRUE, ...) {
     ll <- lu(x, errSing = FALSE)
     ##          ^^^^^^^^^^^^^^^ no error in case of singularity
     if(identical(NA, ll)) { ## LU-decomposition failed with singularity
-	return(mkDet(ldet=if(any(is.na(x))) NaN else -Inf,
+	return(mkDet(ldet = if(anyNA(x)) NaN else -Inf,
 		     logarithm=logarithm, sig = 1L))
     }
     ## else
@@ -251,6 +277,9 @@ mkDet <- function(d, logarithm = TRUE, ldet = sum(log(abs(d))),
     val
 }
 
+##' utility, basically == norm(x, type = "2")
+norm2 <- function(x) if(anyNA(x)) NaN else svd(x, nu = 0L, nv = 0L)$d[1L]
+
 dimCheck <- function(a, b) {
     da <- dim(a)
     db <- dim(b)
@@ -288,32 +317,84 @@ mmultCheck <- function(a, b, kind = 1L) {
     ca
 }
 
-dimNamesCheck <- function(a, b) {
-    ## Constructs "sensical" dimnames for something like  a + b ;
-    ## assume dimCheck() has happened before
+##' Constructs "sensical" dimnames for something like  a + b ;
+##' assume dimCheck() has happened before
+##'
+##' NOTA BENE:   R's  ?Arithmetic  says
+##' ---------
+##'>  For arrays (and an array result) the dimensions and dimnames are taken from
+##'>  first argument if it is an array, otherwise the second.
+##' but that's not quite correct:
+##' The dimnames are taken from second *if* the first are NULL.
+##'
+##' @title Construct dimnames for  a  o  b
+##' @param a matrix
+##' @param b matrix
+##' @param useFirst logical indicating if dimnames(a), the first, is taken, unless NULL
+##' @param check logical indicating if a warning should be signalled for mismatches
+##' @return a \code{\link{list}} of length two with dimnames
+##' @author Martin Maechler
+dimNamesCheck <- function(a, b, useFirst = TRUE, check = FALSE) {
     nullDN <- list(NULL,NULL)
     h.a <- !identical(nullDN, dna <- dimnames(a))
     h.b <- !identical(nullDN, dnb <- dimnames(b))
     if(h.a || h.b) {
-	if (!h.b) dna
-	else if(!h.a) dnb
-	else { ## both have non-trivial dimnames
-	    r <- dna # "default" result
-	    for(j in 1:2) if(!is.null(dn <- dnb[[j]])) {
-		if(is.null(r[[j]]))
-		    r[[j]] <- dn
-		else if(!identical(r[[j]], dn))
-		    warning(gettextf("dimnames [%d] mismatch in %s", j,
-				     deparse(sys.call(sys.parent()))),
-			    call. = FALSE, domain=NA)
-	    }
-	    r
-	}
+        if(useFirst) {
+            if(!h.a) dnb else dna
+        } else {
+            if (!h.b) dna
+            else if(!h.a) dnb
+            else { ## both have non-trivial dimnames
+                r <- dna # "default" result
+                for(j in 1:2) if(!is.null(dn <- dnb[[j]])) {
+                    if(is.null(r[[j]]))
+                        r[[j]] <- dn
+                    else if(check && !identical(r[[j]], dn))
+                        warning(gettextf("dimnames [%d] mismatch in %s", j,
+                                         deparse(sys.call(sys.parent()))),
+                                call. = FALSE, domain=NA)
+                }
+                r
+            }
+        }
     }
     else
 	nullDN
 }
 
+##' @title Symmetrize dimnames(.)
+##' @param x a square matrix
+##' @param col logical indicating if the column names should be taken when
+##' both are non-NULL.
+##' @param names logical indicating if the names(dimnames(.)) should be
+##' symmetrized and kept *if* they differ.
+##' @return a matrix like \code{x}, say \code{r}, with dimnames fulfilling
+##' 		dr <- dimnames(r); identical(dr[1], dr[2])
+##' @author Martin Maechler
+symmetrizeDimnames <- function(x, col=TRUE, names=TRUE) {
+    dimnames(x) <- symmDN(dimnames(x), col=col, names=names)
+    x
+}
+
+symmDN <- function(dn, col=TRUE, names=TRUE) {
+    if(is.null(dn) || identical(dn[1L], dn[2L]))
+	return(dn)
+    J <-
+        if(col) {
+            if(is.null(dn[[2L]])) 1L else 2L
+        } else { ## !col : row
+            if(is.null(dn[[1L]])) 2L else 1L
+        }
+
+    if(!is.null(n <- names(dn))) {
+	if(length(n) != 2)
+	    stop("names(dimnames(<matrix>)) must be NULL or of length two")
+	if(n[1L] != n[2L])
+	    names(dn) <- if(names) n[c(J,J)] # else NULL
+    }
+    dn[c(J,J)]
+}
+
 rowCheck <- function(a, b) {
     da <- dim(a)
     db <- dim(b)
@@ -375,10 +456,10 @@ isPacked <- function(x)
     ## Is 'x' a packed (dense) matrix ?
     is(x, "denseMatrix") &&
     ## unneeded(!): any("x" == slotNames(x)) &&
-    length(x@x) < prod(dim(x))
+    length(x@x) < prod(x@Dim)
 }
 ##" Is 'x' a packed (dense) matrix -- "no-check" version
-.isPacked <- function(x) length(x@x) < prod(dim(x))
+.isPacked <- function(x) length(x@x) < prod(x@Dim)
 
 emptyColnames <- function(x, msg.if.not.empty = FALSE)
 {
@@ -402,6 +483,7 @@ emptyColnames <- function(x, msg.if.not.empty = FALSE)
 }
 
 
+
 ## The i-th unit vector  e[1:n] with e[j] = \delta_{i,j}
 ## .E.i.log <- function(i,n)  i == (1:n)
 ## .E.i <- function(i,n)
@@ -442,7 +524,7 @@ indTri <- function(n, upper = TRUE, diag = FALSE) {
     ## Indices of (strict) upper/lower triangular part
     ## == which(upper.tri(diag(n), diag=diag) or
     ##	  which(lower.tri(diag(n), diag=diag) -- but
-    ## more efficiently for largish 'n'
+    ## much more efficiently for largish 'n'
     stopifnot(length(n) == 1, n == (n. <- as.integer(n)), (n <- n.) >= 0)
     if(n <= 2) {
         if(n == 0) return(integer(0))
@@ -671,22 +753,12 @@ uniqTsparse <- function(x, class.x = c(class(x))) {
     ## and as() are based on C code  (all of them?)
     ##
     ## FIXME: Do it fast for the case where 'x' is already 'uniq'
-
-    switch(class.x,
-	   "dgTMatrix" = as(as(x, "dgCMatrix"), "dgTMatrix"),
-	   "dsTMatrix" = as(as(x, "dsCMatrix"), "dsTMatrix"),
-	   "dtTMatrix" = as(as(x, "dtCMatrix"), "dtTMatrix"),
-	   ## do we need this for "logical" ones, there's no sum() there!
-	   "lgTMatrix" = as(as(x, "lgCMatrix"), "lgTMatrix"),
-	   "lsTMatrix" = as(as(x, "lsCMatrix"), "lsTMatrix"),
-	   "ltTMatrix" = as(as(x, "ltCMatrix"), "ltTMatrix"),
-	   ## do we need this for "logical" ones, there's no sum() there!
-	   "ngTMatrix" = as(as(x, "ngCMatrix"), "ngTMatrix"),
-	   "nsTMatrix" = as(as(x, "nsCMatrix"), "nsTMatrix"),
-	   "ntTMatrix" = as(as(x, "ntCMatrix"), "ntTMatrix"),
-	   ## otherwise:
-	   stop(gettextf("not yet implemented for class %s", dQuote(class.x)),
-		domain = NA))
+    if(extends(class.x, "TsparseMatrix")) {
+	tri <- extends(class.x, "triangularMatrix")
+	.Call(Csparse_to_Tsparse, .Call(Tsparse_to_Csparse, x, tri), tri)
+    } else
+	stop(gettextf("not yet implemented for class %s", dQuote(class.x)),
+	     domain = NA)
 }
 
 ## Note: maybe, using
@@ -694,15 +766,14 @@ uniqTsparse <- function(x, class.x = c(class(x))) {
 ## would be slightly more efficient than as( <dgC> , "dgTMatrix")
 ## but really efficient would be to use only one .Call(.) for uniq(.) !
 
-## Earlier was:
-## drop0 <- function(x, clx = c(class(x)), tol = 0) {
 drop0 <- function(x, tol = 0, is.Csparse = NA) {
     .Call(Csparse_drop,
-	  if(isTRUE(is.Csparse) || is.na(is.Csparse) && is(x, "CsparseMatrix")) x else
-          as(x, "CsparseMatrix"),
+	  if(isTRUE(is.Csparse) || is.na(is.Csparse) && is(x, "CsparseMatrix"))
+	      x else as(x, "CsparseMatrix"),
 	  tol)
 }
 
+
 uniq <- function(x) {
     if(is(x, "TsparseMatrix")) uniqTsparse(x) else
     if(is(x, "sparseMatrix")) drop0(x) else x
@@ -714,10 +785,11 @@ asTuniq <- function(x) {
 
 ## is 'x' a uniq Tsparse Matrix ?
 is_not_uniqT <- function(x, di = dim(x))
-    is.unsorted(x@j) || anyDuplicated(.Call(m_encodeInd2, x@i, x@j, di, FALSE, FALSE))
+    is.unsorted(x@j) || anyDuplicatedT(x, di)
 
 ## is 'x' a TsparseMatrix with duplicated entries (to be *added* for uniq):
-is_duplicatedT <- function(x, di = dim(x))
+is_duplicatedT <- # <- keep old name for a while, as ../inst/test-tools-Matrix.R has used it
+anyDuplicatedT <- function(x, di = dim(x))
     anyDuplicated(.Call(m_encodeInd2, x@i, x@j, di, FALSE, FALSE))
 
 
@@ -749,6 +821,16 @@ fixupDense <- function(m, from, cldm = getClassDef(class(m))) {
     m
 }
 
+##' @title Transform {vectors, matrix, Matrix, ...} to  dgeMatrix
+##' @export
+..2dge <- function(from) .Call(dup_mMatrix_as_dgeMatrix, from)
+if(FALSE) ## FIXME: From R we want something like (but all in C - where inherits() is "free"
+..2dge <- function(from, check=TRUE) {
+    if(check && inherits(from,"geMatrix")) from
+    else .Call(dup_mMatrix_as_dgeMatrix, from)
+}
+
+
 ## -> ./ldenseMatrix.R :
 l2d_Matrix <- function(from, cl = MatrixClass(class(from)), cld = getClassDef(cl)) {
     ## stopifnot(is(from, "lMatrix"))
@@ -818,8 +900,12 @@ tT2gT <- function(x, cl = class(x), toClass, cld = getClassDef(cl)) {
 ## Just *modify* the 'x' object , using attr(x, "class') <- toClass
 
 
-## Fast very special one
-## .gT2tC <- function(x, uplo, diag) .Call(Tsparse_to_tCsparse, x, uplo, diag)
+## Fast very special one ../src/Tsparse.c -- as_cholmod_triplet() in ../src/chm_common.c
+## 'x' *must* inherit from TsparseMatrix!
+.gT2tC <- function(x, uplo, diag="N") .Call(Tsparse_to_tCsparse, x, uplo, diag)
+## Ditto in ../src/Csparse.c :
+.gC2tC <- function(x, uplo, diag="N") .Call(Csparse_to_tCsparse, x, uplo, diag)
+.gC2tT <- function(x, uplo, diag="N") .Call(Csparse_to_tTsparse, x, uplo, diag)
 
 gT2tT <- function(x, uplo, diag, toClass,
 		  do.n = extends(toClass, "nMatrix"))
@@ -845,7 +931,7 @@ gT2tT <- function(x, uplo, diag, toClass,
 
 check.gT2tT <- function(from, toClass, do.n = extends(toClass, "nMatrix")) {
     if(isTr <- isTriangular(from)) {
-	gT2tT(from, uplo = .if.NULL(attr(isTr, "kind"), "U"),
+	gT2tT(from, uplo = attr(isTr, "kind") %||% "U",
 	      diag = "N", ## improve: also test for unit diagonal
 	      toClass = toClass, do.n = do.n)
     } else stop("not a triangular matrix")
@@ -905,17 +991,25 @@ l2d_meth <- function(x) {
     else .M.kindC(clx)
 }
 
-.M.kindC <- function(clx) { ## 'clx': class() *or* classdefinition
+.M.kindC <- function(clx, ex = extends(clx)) { ## 'clx': class() *or* classdefinition
     if(is.character(clx))		# < speedup: get it once
-        clx <- getClassDef(clx)
-    if(extends(clx, "sparseVector")) ## shortcut
-	substr(as.character(clx@className), 1,1)
-    else if(extends(clx, "dMatrix")) "d"
-    else if(extends(clx, "nMatrix")) "n"
-    else if(extends(clx, "lMatrix")) "l"
-    else if(extends(clx, "indMatrix")) "n" # permutation -> pattern
-    else if(extends(clx, "zMatrix")) "z"
-    else if(extends(clx, "iMatrix")) "i"
+	clx <- getClassDef(clx)
+    if(any(ex == "sparseVector")) {
+	## must work for class *extending* "dsparseVector" ==> cannot use  (clx@className) !
+	if     (any(ex == "dsparseVector")) "d"
+	else if(any(ex == "nsparseVector")) "n"
+	else if(any(ex == "lsparseVector")) "l"
+	else if(any(ex == "zsparseVector")) "z"
+	else if(any(ex == "isparseVector")) "i"
+	else stop(gettextf(" not yet implemented for %s", clx@className),
+		  domain = NA)
+    }
+    else if(any(ex == "dMatrix")) "d"
+    else if(any(ex == "nMatrix")) "n"
+    else if(any(ex == "lMatrix")) "l"
+    else if(any(ex == "indMatrix")) "n" # permutation -> pattern
+    else if(any(ex == "zMatrix")) "z"
+    else if(any(ex == "iMatrix")) "i"
     else stop(gettextf(" not yet implemented for %s", clx@className),
 	      domain = NA)
 }
@@ -927,7 +1021,7 @@ l2d_meth <- function(x) {
 		"n" = "logical",
 		"z" = "complex")
 ## the reverse, a "version of" .M.kind(.):
-.kind.type <- names(.type.kind); names(.kind.type) <- as.vector(.type.kind)
+.kind.type <- setNames(names(.type.kind), as.vector(.type.kind))
 
 .M.shape <- function(x, clx = class(x)) {
     ## 'clx': class() *or* class definition of x
@@ -940,9 +1034,10 @@ l2d_meth <- function(x) {
     else {
 	if(is.character(clx)) # < speedup: get it once
 	    clx <- getClassDef(clx)
-	if(extends(clx, "diagonalMatrix"))  "d"
-	else if(extends(clx, "triangularMatrix"))"t"
-	else if(extends(clx, "symmetricMatrix")) "s"
+        ex <- extends(clx)
+	if(     any(ex == "diagonalMatrix"))  "d"
+	else if(any(ex == "triangularMatrix"))"t"
+	else if(any(ex == "symmetricMatrix")) "s"
 	else "g"
     }
 }
@@ -1043,8 +1138,6 @@ as_Csp2 <- function(x) {
     if(is(x, "triangularMatrix")) .Call(Csparse_diagU2N, x) else x
 }
 
-.gC2sym <- function(x, uplo) .Call(Csparse_general_to_symmetric, x, uplo)
-
 ## 'cl'   : class() *or* class definition of from
 as_gCsimpl2 <- function(from, cl = class(from))
     as(from, paste0(.M.kind(from, cl), "gCMatrix"))
@@ -1091,7 +1184,7 @@ as_CspClass <- function(x, cl) {
     ##(extends(cld, "diagonalMatrix") && isDiagonal(x)) ||
     if (extends(cld, "symmetricMatrix") && isSymmetric(x))
         forceSymmetric(as(x,"CsparseMatrix"))
-    else if (extends(cld, "triangularMatrix") && (iT <- isTriangular(x)))
+    else if (extends(cld, "triangularMatrix") && isTriangular(x))
 	as(x, cl)
     else if(is(x, "CsparseMatrix")) x
     else as(x, paste0(.M.kind(x, cld), "gCMatrix"))
@@ -1106,6 +1199,20 @@ asTri <- function(from, newclass) {
     else stop("not a triangular matrix")
 }
 
+mat2tri <- function(from, sparse=NA) {
+    isTri <- isTriangular(from)
+    if(isTri) {
+	d <- dim(from)
+	if(is.na(sparse))
+	    sparse <- prod(d) > 2*sum(isN0(from)) ## <==> sparseDefault() in Matrix()
+	if(sparse)
+	    as(as(from, "sparseMatrix"), "triangularMatrix")
+	else
+	    new(paste0(.M.kind(from),"trMatrix"), x = base::as.vector(from),
+                Dim = d, Dimnames = .M.DN(from), uplo = attr(isTri, "kind"))
+    }
+    else stop("not a triangular matrix")
+}
 
 
 try_as <- function(x, classes, tryAnyway = FALSE) {
@@ -1121,7 +1228,7 @@ try_as <- function(x, classes, tryAnyway = FALSE) {
 
 
 ## For *dense* matrices
-isTriMat <- function(object, upper = NA) {
+isTriMat <- function(object, upper = NA, ...) {
     ## pretest: is it square?
     d <- dim(object)
     if(d[1] != d[2]) return(FALSE)
@@ -1144,7 +1251,7 @@ isTriMat <- function(object, upper = NA) {
 }
 
 ## For Tsparse matrices:
-isTriT <- function(object, upper = NA) {
+isTriT <- function(object, upper = NA, ...) {
     ## pretest: is it square?
     d <- dim(object)
     if(d[1] != d[2]) return(FALSE)
@@ -1166,7 +1273,7 @@ isTriT <- function(object, upper = NA) {
 }
 
 ## For Csparse matrices
-isTriC <- function(object, upper = NA) {
+isTriC <- function(object, upper = NA, ...) {
     ## pretest: is it square?
     d <- dim(object)
     if(d[1] != d[2]) return(FALSE)
@@ -1183,7 +1290,7 @@ isTriC <- function(object, upper = NA) {
     ni <- 1:n
     ## the row indices split according to column:
     ilist <- split(object@i, factor(rep.int(ni, diff(object@p)), levels= ni))
-    lil <- unlist(lapply(ilist, length), use.names = FALSE)
+    lil <- lengths(ilist, use.names = FALSE)
     if(any(lil == 0)) {
 	pos <- lil > 0
 	if(!any(pos)) ## matrix of all 0's
@@ -1225,14 +1332,12 @@ isTriC <- function(object, upper = NA) {
             ## -> has 'uplo'  differentiate between packed and unpacked
 
 ### .......... FIXME ...............
-
-            packed <- isPacked(object)
-            if(object@uplo == "U") {
-            } else { ## uplo == "L"
-            }
-
+	    ## packed <- isPacked(object)
+	    ## if(object@uplo == "U") {
+	    ## } else { ## uplo == "L"
+	    ## }
 ### very cheap workaround
-	    all0(as.matrix(object)[rep_len(c(FALSE, rep.int(TRUE,n)), n^2)])
+	    all0(as(object,"matrix")[rep_len(c(FALSE, rep.int(TRUE,n)), n^2)])
         }
 }
 
@@ -1397,11 +1502,10 @@ all0Matrix <- function(n,m) {
 	p = rep.int(0L, m+1L))
 }
 
-setZero <- function(x) {
+.setZero <- function(x, newclass = if(d[1] == d[2]) "dsCMatrix" else "dgCMatrix") {
     ## all-0 matrix  from x  which must inherit from 'Matrix'
     d <- x@Dim
-    new(if(d[1] == d[2]) "dsCMatrix" else "dgCMatrix",
-	Dim = d, Dimnames = x@Dimnames, p = rep.int(0L, d[2]+1L))
+    new(newclass, Dim = d, Dimnames = x@Dimnames, p = rep.int(0L, d[2]+1L))
 }
 
 .M.vectorSub <- function(x,i) { ## e.g. M[0] , M[TRUE],  M[1:2]
@@ -1443,7 +1547,7 @@ setparts <- function(x,y, uniqueCheck = TRUE, check = TRUE) {
 }
 
 ##' @title Warn about extraneous arguments in the "..."  (of its caller).
-##' A merger of my approach and the one in seq.default()
+##' A merger of my approach and the one in seq.default() -- FIXME: now have base::chkDots()
 ##' @author Martin Maechler, June 2012, May 2014
 ##' @param ...
 ##' @param which.call passed to sys.call().  A caller may use -2 if the message should
@@ -1459,7 +1563,7 @@ chk.s <- function(..., which.call = -1) {
 }
 
 ##' *Only* to be used as function in
-##'    setMethod("Compare", ...., .Cmp.swap)  -->  ./Ops.R  & ./diagMatrix.R
+##'    setMethod.("Compare", ...., .Cmp.swap)  -->  ./Ops.R  & ./diagMatrix.R
 .Cmp.swap <- function(e1,e2) {
     ## "swap RHS and LHS" and use the method below:
     switch(.Generic,
